/*
 * Operation.java
 *
 * Idea taken from org.jfree.util.SortOrder
 */

package de.pdv_systeme.DataCollection;

/**
 * A class to define calculation operation constants.
 *
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public final class Operation {
    
    /** Operation for {@link AbstractDataCollection#addCalculatedSeries addCalculatedSeries} */
    public static final Operation
            NONE     = new Operation("Operation.NONE"),
            ADD      = new Operation("Operation.ADD"),
            SUBTRACT = new Operation("Operation.SUBTRACT"),
            MULTIPLY = new Operation("Operation.MULTIPLY"),
            DIVIDE   = new Operation("Operation.DIVIDE");
    
    /** The name. */
    private String name;
    
    /**
     * Private constructor.
     *
     * @param name  the name.
     */
    private Operation(final String name) {
        this.name = name;
    }
    
    /**
     * Calculate a value.
     * @param d1  the first operand
     * @param d2  the second operand
     * @return the result
     */
    public Double execute(final double d1, final double d2) {
        if (this == ADD)
            return new Double(d1 + d2);
        else if (this == SUBTRACT)
            return new Double(d1 - d2);
        else if (this == MULTIPLY)
            return new Double(d1 * d2);
        else if (this == DIVIDE) {
            if (d2 != 0.0)
                return new Double(d1 / d2);
            else
                throw new ArithmeticException("Division by zero");
        }
        return null;
    }
    
    /**
     * Return a string representation of the operation.
     *
     * @return The string
     */
    public String toString() {
        return this.name;
    }
    
    /**
     * Returns <code>true</code> if this object is equal to the specified 
     * object, and <code>false</code> otherwise.
     *
     * @param obj  the other object.
     *
     * @return A boolean.
     */
    public boolean equals(final Object obj) {

        if (this == obj) {
            return true;
        }
        if (! (obj instanceof Operation)) {
            return false;
        }

        final Operation that = (Operation) obj;
        if (! this.name.equals(that.toString())) {
            return false;
        }

        return true;
    }
    
    /**
     * Returns a hash code value for the object.
     *
     * @return The hashcode
     */
    public int hashCode() {
        return this.name.hashCode();
    }

}
