/*
 * ChartPropertiesDialog.java
 */

package de.pdv_systeme.JTLviz;

import java.awt.Rectangle;
import java.util.Date;

import java.awt.Color;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;

import org.jfree.chart.JFreeChart;

import de.pdv_systeme.DataCollection.AbstractDataCollection;
import de.pdv_systeme.util.VmsDate;

/**
 * Sets up a (modal) chart properties dialog
 *
 * @author  Martin Vorlnder <mv@pdv-systeme.de>
 */
public class ChartPropertiesDialog
        extends JDialog {
    
    // TODO: Implement all the tabs besides the data table (re-implement Teechart? ;-)

    /* Action command string for navigation in the data table */
    private static final String
            NAV_FIRST = "First",
            NAV_PRIOR = "Prior",
            NAV_NEXT  = "Next",
            NAV_LAST  = "Last";

    /** The dialog's parent frame */
    private JFrame parent;

    /** The associated chart */
    private JFreeChart chart;

    /** The data table's model */
    private ChartPropertiesDataTable dataTable;
    
    /**
     * Creates new form ChartPropertiesDialog
     * @param parent  the dialog's parent frame
     * @param chart  the associated chart
     */
    public ChartPropertiesDialog(final JFrame parent, final JFreeChart chart) {
        super(parent, true);
        
        if (chart == null)
            throw new IllegalArgumentException("Null 'chart' argument");
        
        this.parent = parent;
        this.chart = chart;

        initComponents();
    }
    
    /** Receives notifications of selection changes to the data table. */
    private class MyDataTableSelectionListener implements ListSelectionListener {
        /** {@inheritDoc} */
        public void valueChanged(final ListSelectionEvent e) {
            if (! e.getValueIsAdjusting()) {
                ListSelectionModel lsm = (ListSelectionModel) e.getSource();
                final boolean ROW_SELECTED = ! lsm.isSelectionEmpty();
                final int selectedRow = ROW_SELECTED ? lsm.getMinSelectionIndex() : -1;
                final int firstRow = 0, lastRow = dataTable.getModel().getRowCount()-1;
                final boolean NOT_FIRST_ROW = (selectedRow != firstRow);
                final boolean NOT_LAST_ROW = (selectedRow != lastRow);

                dataFirstButton.setEnabled (NOT_FIRST_ROW);
                dataPriorButton.setEnabled (NOT_FIRST_ROW);
                dataNextButton.setEnabled  (NOT_LAST_ROW);
                dataLastButton.setEnabled  (NOT_LAST_ROW);
                // TODO: Implement editing the table data (and changing the underlying time series)
                dataInsertButton.setEnabled(false); // ROW_SELECTED
                dataDeleteButton.setEnabled(false); // ROW_SELECTED
                dataEditButton.setEnabled  (false); // ROW_SELECTED
                dataPostButton.setEnabled  (false); // dataTableModified
                dataCancelButton.setEnabled(false); // dataTableModified
            }
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JPanel dataButtonsPanel;
        javax.swing.JPanel jPanel1;
        javax.swing.JPanel jPanel2;

        topTabbedPane = new javax.swing.JTabbedPane();
        chartTabbedPane = new javax.swing.JTabbedPane();
        chartSeriesPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        chartGeneralPanel = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        chartGeneralZoomPanel = new javax.swing.JPanel();
        chartGeneralScrollPanel = new javax.swing.JPanel();
        chartAxisPanel = new javax.swing.JPanel();
        chartTitlesPanel = new javax.swing.JPanel();
        chartLegendTabbedPane = new javax.swing.JTabbedPane();
        chartLegendStylePanel = new javax.swing.JPanel();
        chartLegendPositionPanel = new javax.swing.JPanel();
        chartLegendSymbolsPanel = new javax.swing.JPanel();
        chartLegendFormatPanel = new javax.swing.JPanel();
        chartLegendTextPanel = new javax.swing.JPanel();
        chartLegendGradientPanel = new javax.swing.JPanel();
        chartLegendShadowPanel = new javax.swing.JPanel();
        chartPanelTabbedPane = new javax.swing.JTabbedPane();
        chartPanelBackgroundPanel = new javax.swing.JPanel();
        chartPanelBordersPanel = new javax.swing.JPanel();
        chartPanelGradientPanel = new javax.swing.JPanel();
        chartPagingPanel = new javax.swing.JPanel();
        chartWallsPanel = new javax.swing.JPanel();
        chart3DPanel = new javax.swing.JPanel();
        seriesPanel = new javax.swing.JPanel();
        dataPanel = new javax.swing.JPanel();
        dataTableScrollPane = new javax.swing.JScrollPane();
        dataButtonsPanel = new javax.swing.JPanel();
        dataFirstButton = new javax.swing.JButton();
        dataPriorButton = new javax.swing.JButton();
        dataNextButton = new javax.swing.JButton();
        dataLastButton = new javax.swing.JButton();
        dataInsertButton = new javax.swing.JButton();
        dataDeleteButton = new javax.swing.JButton();
        dataEditButton = new javax.swing.JButton();
        dataPostButton = new javax.swing.JButton();
        dataCancelButton = new javax.swing.JButton();
        toolsPanel = new javax.swing.JPanel();
        exportPanel = new javax.swing.JPanel();
        printPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        helpButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        chartSeriesPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 20));

        jLabel1.setText(Main.getResourceString("NYI_(except_for_the_Data_Panel)"));
        chartSeriesPanel.add(jLabel1);

        chartTabbedPane.addTab(Main.getResourceString("Series"), chartSeriesPanel);

        chartGeneralPanel.setLayout(new java.awt.BorderLayout());

        chartGeneralPanel.add(jPanel2, java.awt.BorderLayout.WEST);

        jTabbedPane1.addTab(Main.getResourceString("Zoom"), chartGeneralZoomPanel);

        jTabbedPane1.addTab(Main.getResourceString("Scroll"), chartGeneralScrollPanel);

        chartGeneralPanel.add(jTabbedPane1, java.awt.BorderLayout.CENTER);

        chartTabbedPane.addTab(Main.getResourceString("General"), chartGeneralPanel);

        chartTabbedPane.addTab(Main.getResourceString("Axis"), chartAxisPanel);

        chartTabbedPane.addTab(Main.getResourceString("Titles"), chartTitlesPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Style"), chartLegendStylePanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Position"), chartLegendPositionPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Symbols"), chartLegendSymbolsPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Format"), chartLegendFormatPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Text"), chartLegendTextPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Legend_Gradient"), chartLegendGradientPanel);

        chartLegendTabbedPane.addTab(Main.getResourceString("Shadow"), chartLegendShadowPanel);

        chartTabbedPane.addTab(Main.getResourceString("Legend"), chartLegendTabbedPane);

        chartPanelTabbedPane.addTab(Main.getResourceString("Background"), chartPanelBackgroundPanel);

        chartPanelTabbedPane.addTab(Main.getResourceString("Borders"), chartPanelBordersPanel);

        chartPanelTabbedPane.addTab(Main.getResourceString("Panel_Gradient"), chartPanelGradientPanel);

        chartTabbedPane.addTab(Main.getResourceString("Panel"), chartPanelTabbedPane);

        chartTabbedPane.addTab(Main.getResourceString("Paging"), chartPagingPanel);

        chartTabbedPane.addTab(Main.getResourceString("Walls"), chartWallsPanel);

        chartTabbedPane.addTab(Main.getResourceString("3D"), chart3DPanel);

        topTabbedPane.addTab(Main.getResourceString("Chart"), chartTabbedPane);

        topTabbedPane.addTab(Main.getResourceString("Series"), seriesPanel);

        dataPanel.setLayout(new java.awt.BorderLayout());

        dataTableScrollPane.setAutoscrolls(true);
        dataTable = new ChartPropertiesDataTable(chart.getXYPlot());
        dataTable.getSelectionModel().addListSelectionListener(
            new MyDataTableSelectionListener());
        dataTableScrollPane.setViewportView(dataTable);

        dataPanel.add(dataTableScrollPane, java.awt.BorderLayout.CENTER);

        dataButtonsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 0, 5));

        java.awt.Insets buttonInset = new java.awt.Insets(2, 4, 2, 4);
        dataFirstButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/First.gif")));
        dataFirstButton.setToolTipText(Main.getResourceString("First"));
        dataFirstButton.setActionCommand(NAV_FIRST);
        dataFirstButton.setBorderPainted(false);
        dataFirstButton.setContentAreaFilled(false);
        dataFirstButton.setMargin(buttonInset);
        dataFirstButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dataNavigationActionPerformed(evt);
            }
        });

        dataButtonsPanel.add(dataFirstButton);

        dataPriorButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Prior.gif")));
        dataPriorButton.setToolTipText(Main.getResourceString("Prior"));
        dataPriorButton.setActionCommand(NAV_PRIOR);
        dataPriorButton.setBorderPainted(false);
        dataPriorButton.setContentAreaFilled(false);
        dataPriorButton.setMargin(buttonInset);
        dataPriorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dataNavigationActionPerformed(evt);
            }
        });

        dataButtonsPanel.add(dataPriorButton);

        dataNextButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Next.gif")));
        dataNextButton.setToolTipText(Main.getResourceString("Next"));
        dataNextButton.setActionCommand(NAV_NEXT);
        dataNextButton.setBorderPainted(false);
        dataNextButton.setContentAreaFilled(false);
        dataNextButton.setMargin(buttonInset);
        dataNextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dataNavigationActionPerformed(evt);
            }
        });

        dataButtonsPanel.add(dataNextButton);

        dataLastButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Last.gif")));
        dataLastButton.setToolTipText(Main.getResourceString("Last"));
        dataLastButton.setActionCommand(NAV_LAST);
        dataLastButton.setBorderPainted(false);
        dataLastButton.setContentAreaFilled(false);
        dataLastButton.setMargin(buttonInset);
        dataLastButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dataNavigationActionPerformed(evt);
            }
        });

        dataButtonsPanel.add(dataLastButton);

        dataInsertButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Insert.gif")));
        dataInsertButton.setToolTipText(Main.getResourceString("Insert"));
        dataInsertButton.setBorderPainted(false);
        dataInsertButton.setContentAreaFilled(false);
        dataInsertButton.setEnabled(false);
        dataInsertButton.setMargin(buttonInset);
        dataButtonsPanel.add(dataInsertButton);

        dataDeleteButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Delete.gif")));
        dataDeleteButton.setToolTipText(Main.getResourceString("Delete"));
        dataDeleteButton.setBorderPainted(false);
        dataDeleteButton.setContentAreaFilled(false);
        dataDeleteButton.setEnabled(false);
        dataDeleteButton.setMargin(buttonInset);
        dataButtonsPanel.add(dataDeleteButton);

        dataEditButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Edit.gif")));
        dataEditButton.setToolTipText(Main.getResourceString("Edit"));
        dataEditButton.setBorderPainted(false);
        dataEditButton.setContentAreaFilled(false);
        dataEditButton.setEnabled(false);
        dataEditButton.setMargin(buttonInset);
        dataButtonsPanel.add(dataEditButton);

        dataPostButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Post.gif")));
        dataPostButton.setToolTipText(Main.getResourceString("Post"));
        dataPostButton.setBorderPainted(false);
        dataPostButton.setContentAreaFilled(false);
        dataPostButton.setEnabled(false);
        dataPostButton.setMargin(buttonInset);
        dataButtonsPanel.add(dataPostButton);

        dataCancelButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/pdv_systeme/JTLviz/Images/Cancel.gif")));
        dataCancelButton.setToolTipText(Main.getResourceString("Cancel"));
        dataCancelButton.setBorderPainted(false);
        dataCancelButton.setContentAreaFilled(false);
        dataCancelButton.setEnabled(false);
        dataCancelButton.setMargin(buttonInset);
        dataButtonsPanel.add(dataCancelButton);

        dataPanel.add(dataButtonsPanel, java.awt.BorderLayout.SOUTH);

        topTabbedPane.addTab(Main.getResourceString("Data"), dataPanel);

        topTabbedPane.addTab(Main.getResourceString("Tools"), toolsPanel);

        topTabbedPane.addTab(Main.getResourceString("Export"), exportPanel);

        topTabbedPane.addTab(Main.getResourceString("Print"), printPanel);

        getContentPane().add(topTabbedPane, java.awt.BorderLayout.CENTER);

        helpButton.setText(Main.getResourceString("Help..."));
        helpButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpButtonActionPerformed(evt);
            }
        });

        jPanel1.add(helpButton);

        closeButton.setText(Main.getResourceString("Close"));
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        jPanel1.add(closeButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents

    private void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpButtonActionPerformed
        // TODO: implement "Chart Properties Help"
        Main.showNYI(parent);
    }//GEN-LAST:event_helpButtonActionPerformed
    
    private void dataNavigationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dataNavigationActionPerformed
        String action = evt.getActionCommand();
        final int firstRow = 0, lastRow = dataTable.getModel().getRowCount()-1;
        int selectedRow = dataTable.getSelectedRow();

        if (action == NAV_FIRST) {
            selectedRow = firstRow;
        } else if (action == NAV_PRIOR && selectedRow > firstRow) {
            selectedRow--;
        } else if (action == NAV_NEXT && selectedRow < lastRow) {
            selectedRow++;
        } else if (action == NAV_LAST) {
            selectedRow = lastRow;
        }
        dataTable.setSelectedRow(selectedRow);
    }//GEN-LAST:event_dataNavigationActionPerformed
    
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        dispose();
    }//GEN-LAST:event_closeButtonActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel chart3DPanel;
    private javax.swing.JPanel chartAxisPanel;
    private javax.swing.JPanel chartGeneralPanel;
    private javax.swing.JPanel chartGeneralScrollPanel;
    private javax.swing.JPanel chartGeneralZoomPanel;
    private javax.swing.JPanel chartLegendFormatPanel;
    private javax.swing.JPanel chartLegendGradientPanel;
    private javax.swing.JPanel chartLegendPositionPanel;
    private javax.swing.JPanel chartLegendShadowPanel;
    private javax.swing.JPanel chartLegendStylePanel;
    private javax.swing.JPanel chartLegendSymbolsPanel;
    private javax.swing.JTabbedPane chartLegendTabbedPane;
    private javax.swing.JPanel chartLegendTextPanel;
    private javax.swing.JPanel chartPagingPanel;
    private javax.swing.JPanel chartPanelBackgroundPanel;
    private javax.swing.JPanel chartPanelBordersPanel;
    private javax.swing.JPanel chartPanelGradientPanel;
    private javax.swing.JTabbedPane chartPanelTabbedPane;
    private javax.swing.JPanel chartSeriesPanel;
    private javax.swing.JTabbedPane chartTabbedPane;
    private javax.swing.JPanel chartTitlesPanel;
    private javax.swing.JPanel chartWallsPanel;
    private javax.swing.JButton closeButton;
    private javax.swing.JButton dataCancelButton;
    private javax.swing.JButton dataDeleteButton;
    private javax.swing.JButton dataEditButton;
    private javax.swing.JButton dataFirstButton;
    private javax.swing.JButton dataInsertButton;
    private javax.swing.JButton dataLastButton;
    private javax.swing.JButton dataNextButton;
    private javax.swing.JPanel dataPanel;
    private javax.swing.JButton dataPostButton;
    private javax.swing.JButton dataPriorButton;
    private javax.swing.JScrollPane dataTableScrollPane;
    private javax.swing.JPanel exportPanel;
    private javax.swing.JButton helpButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JPanel printPanel;
    private javax.swing.JPanel seriesPanel;
    private javax.swing.JPanel toolsPanel;
    private javax.swing.JTabbedPane topTabbedPane;
    // End of variables declaration//GEN-END:variables
    
}
