/*
 * LoadTask.java
 */

package de.pdv_systeme.JTLviz;

import java.io.IOException;
import java.text.ParseException;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.Timer;
import javax.swing.UIManager;

import de.pdv_systeme.DataCollection.FileDataCollection;
import de.pdv_systeme.util.SwingWorker;


/**
 * Stolen from the Java Tutorial's ProgressBarDemo.
 * Uses a SwingWorker to perform the time-consuming DataCollection file load task.
 */
public class LoadTask {
    private boolean done = false;
    private boolean cancelled = false;
    
    private FileDataCollection dc = null;
    private FileDataCollection timeDc = null;
    
    private JProgressBar progressBar;
    private JOptionPane progressPane;
    private JDialog progressDialog;
    private Timer timer;
    
    /**
     * Creates a new instance of LoadTask to load a data collection.
     *
     * @param parent  the parent component for displaying the progress dialog
     * @param dataCollection  the data collection
     * @param timeDataCollection  the data collection to provide the timestamps (may be <code>null</code>)
     * @throws IOException
     */
    public LoadTask(
            Component parent,
            FileDataCollection dataCollection,
            FileDataCollection timeDataCollection
            )
    throws IOException {
        dc = dataCollection;
        timeDc = timeDataCollection;

        progressBar = new JProgressBar(0, dc.getLinecount());
        progressPane = new JOptionPane(new Object[] {
            Main.getResourceString("Loading..."), progressBar
        }, JOptionPane.PLAIN_MESSAGE, JOptionPane.DEFAULT_OPTION, null, new Object[] {
            Main.getResourceString("CancelLoad")
        });
        progressDialog = progressPane.createDialog(parent,
                UIManager.getString("ProgressMonitor.progressText")
                );
        
        timer = new Timer(100 /*ms*/, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (done || cancelled) {
                    timer.stop();
                    progressDialog.dispose();
                } else {
                    progressBar.setValue(dc.getCurrentLinecount());
                }
            }
        });
    }
    
    /**
     * Returns true iff the load task completed.
     * @return true iff the load task completed
     */
    public boolean isDone() {
        return done;
    }
    
    /**
     * Called to start the task.
     */
    public void go() {
        final SwingWorker worker = new SwingWorker() {
            public Object construct() {
                done = false;
                cancelled = false;
                Object task = null;
                try {
                    task = new ActualTask();
                } catch (InterruptedException e) {
                    // throw new RuntimeException(e);
                } catch (Exception e) {
                    cancelled = true;
                    throw new RuntimeException(e);
                }
                return task;
            }
        };
        timer.start();
        worker.start();
        progressDialog.show();
        Object value = progressPane.getValue();
        if (Main.getResourceString("CancelLoad").equals(value)) {
            cancelled = true;
            worker.interrupt();
            dc.interruptLoad();
        }
    }
    
    /**
     * The actual long running task.  This runs in a SwingWorker thread.
     */
    class ActualTask {
        ActualTask()
        throws IOException, ParseException, InterruptedException {
            dc.loadData(timeDc);
            done = true;
        }
    }
}
