/*
 * TSChartLegendGraphic.java
 */

package de.pdv_systeme.TSChart;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.AbstractAction;
import javax.swing.JCheckBox;

import org.jfree.chart.block.BlockResult;
import org.jfree.chart.block.EntityBlockParams;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.title.LegendGraphic;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.Size2D;

/**
 * A subclass of LegendGraphic to display a checkbox.
 *
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class TSChartLegendGraphic
        extends LegendGraphic {
    
    /**
     * The shape must be non-null for a LegendGraphic - if no shape is required,
     * use this.
     */
    private static final Shape UNUSED_SHAPE = new Line2D.Float();
    
    /**
     * The paint must be non-null for a LegendGraphic - if no paint is required,
     * use this.
     */
    private static final Paint UNUSED_PAINT = Color.black;
    
    JCheckBox checkBox;
    
    /**
     * Creates a new legend graphic.
     *
     * @param item  the legend item for which to create the graphic
     */
    public TSChartLegendGraphic(TSChartLegendItem item) {
        super(UNUSED_SHAPE, UNUSED_PAINT);
        setShapeVisible(false);
        setShapeOutlineVisible(false);
        setLineVisible(false);
        
        checkBox = new JCheckBox(new LegendGraphicAction(item.getLabel()));
        checkBox.setText(null);
        checkBox.setBounds(0, 0, 20, 12);
        checkBox.setFocusable(true);
        checkBox.setEnabled(true);
        checkBox.setSelected(item.isSeriesVisible());
    }
    
    /**
     * Action associated with the legend graphic checkbox
     */
    private class LegendGraphicAction extends AbstractAction {
        public LegendGraphicAction(String name) {
            super(name);
            putValue(SHORT_DESCRIPTION, null);
            putValue(LONG_DESCRIPTION, null);
            putValue(SMALL_ICON, null);
        }
        /** {@inheritDoc} */
        public void actionPerformed(ActionEvent e) {
            checkBox.setSelected(! checkBox.isSelected());
        }
    }
    
    /** {@inheritDoc} */
    protected Size2D arrangeNN(Graphics2D g2) {
        Rectangle2D contentSize = new Rectangle2D.Double();
        if (checkBox != null) {
            contentSize.setRect(checkBox.getBounds());
        }
        return new Size2D(contentSize.getWidth(), contentSize.getHeight());
    }
    
    /** {@inheritDoc} */
    public void draw(Graphics2D g2, Rectangle2D area) {
        area = trimMargin(area);
        drawBorder(g2, area);
        area = trimBorder(area);
        area = trimPadding(area);
        
        Point2D location = RectangleAnchor.coordinates(
                area, RectangleAnchor.CENTER
                );
        checkBox.setLocation((int) location.getX(), (int) location.getY());
        checkBox.paint(g2);
    }
    
    /** {@inheritDoc} */
    public Object draw(Graphics2D g2, Rectangle2D area, Object params) {
        area = trimMargin(area);
        drawBorder(g2, area);
        area = trimBorder(area);
        area = trimPadding(area);

        // check if we need to collect chart entities from the container
        EntityBlockParams ebp = null;
        StandardEntityCollection sec = null;
        Shape entityArea = null;
        if (params instanceof EntityBlockParams) {
            ebp = (EntityBlockParams) params;
            if (ebp.getGenerateEntities()) {
                sec = new StandardEntityCollection();
                // TODO:  this transformation doesn't work always.  Fix!
                entityArea = g2.getTransform().createTransformedShape(area);
            }
        }
        
        Point2D location = RectangleAnchor.coordinates(
                area, RectangleAnchor.CENTER
                );
        checkBox.setLocation((int) location.getX(), (int) location.getY());
        checkBox.paint(g2);

        BlockResult result = null;
        if (ebp != null && sec != null) {
            TSChartLegendItemEntity entity = new TSChartLegendItemEntity(entityArea);
            sec.add(entity);
            result = new BlockResult();
            result.setEntityCollection(sec);
        }
        return result;
    }
}
