/*
 * TSChartLegendTitle.java
 */

package de.pdv_systeme.TSChart;

import org.jfree.chart.LegendItem;
import org.jfree.chart.LegendItemSource;
import org.jfree.chart.block.Block;
import org.jfree.chart.block.BlockContainer;
import org.jfree.chart.block.BorderArrangement;
import org.jfree.chart.block.CenterArrangement;
import org.jfree.chart.block.ColumnArrangement;
import org.jfree.chart.block.FlowArrangement;
import org.jfree.chart.title.LegendGraphic;
import org.jfree.chart.title.LegendTitle;

/**
 * A subclass of LegendTitle to gain control of the LegendTitle's color and graphics.
 *
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class TSChartLegendTitle extends LegendTitle {
    
    /**
     * Constructs a new (empty) legend for the specified source.
     * 
     * @param source  the source.
     */
    public TSChartLegendTitle(LegendItemSource source) {
        super(source, new FlowArrangement(), new ColumnArrangement());
    }
    
    /** {@inheritDoc} */
    protected Block createLegendItemBlock(LegendItem it) {
        TSChartLegendItem item = (TSChartLegendItem) it;

        LegendGraphic lg = new TSChartLegendGraphic(item);
        lg.setShapeFilled(item.isShapeFilled());
        lg.setLine(item.getLine());
        lg.setLineStroke(item.getLineStroke());
        lg.setLinePaint(item.getLinePaint());
        lg.setLineVisible(item.isLineVisible());
        lg.setShapeVisible(item.isShapeVisible());
        lg.setShapeOutlineVisible(item.isShapeOutlineVisible());
        lg.setOutlinePaint(item.getOutlinePaint());
        lg.setOutlineStroke(item.getOutlineStroke());
        lg.setPadding(getLegendItemGraphicPadding());
        lg.setShapeAnchor(getLegendItemGraphicAnchor());
        lg.setShapeLocation(getLegendItemGraphicLocation());

        BlockContainer legendItem = new BlockContainer(new BorderArrangement());
        legendItem.add(lg, getLegendItemGraphicEdge());
        TSChartLabelBlock labelBlock = new TSChartLabelBlock(
                item.getLabel(), item.getLabelFont(), item.getLabelPaint());
        labelBlock.setPadding(getItemLabelPadding());
        labelBlock.setToolTipText(item.getToolTipText());
        legendItem.add(labelBlock);
        
        BlockContainer result = new BlockContainer(new CenterArrangement());
        result.add(legendItem);
        
        return result;
    }
    
}
