/*
 * AboutPanel.java
 */

package de.pdv_systeme.util;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Image;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

/**
 * An About Panel.
 * <p>
 * Basic code stolen from <code>org.jfree.ui.about.AboutPanel</code>.
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class AboutPanel
        extends JPanel {
    
    /**
     * Constructs a panel.
     *
     * @param application  the application name.
     * @param version  the version.
     * @param copyright  the copyright statement.
     * @param info  other info.
     */
    public AboutPanel(
            final String application,
            final String version,
            final String copyright,
            final String[] info) {
        this(application, version, copyright, info, null);
    }
    
    /**
     * Constructs a panel.
     *
     * @param application  the application name.
     * @param version  the version.
     * @param copyright  the copyright statement.
     * @param info  other info.
     * @param logo  an optional logo.
     */
    public AboutPanel(
            final String application,
            final String version,
            final String copyright,
            final String[] info,
            final Image logo) {
        
        setLayout(new BorderLayout());
        
        final JPanel textPanel = new JPanel(new GridLayout(3+info.length, 1, 0, 4));
        
        final JPanel appPanel = new JPanel();
        final Font f1 = new Font("Sans Serif", Font.BOLD, 14);
        final JLabel appLabel = new JLabel(application);
        appLabel.setFont(f1);
        appLabel.setForeground(Color.black);
        appLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        appPanel.add(appLabel);
        textPanel.add(appPanel);
        
        final JPanel verPanel = new JPanel();
        final Font f2 = new Font("Sans Serif", Font.PLAIN, 12);
        final JLabel verLabel = new JLabel(version);
        verLabel.setFont(f2);
        verLabel.setForeground(Color.black);
        verLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        verPanel.add(verLabel);
        textPanel.add(verPanel);
        
        final JPanel copyrightPanel = new JPanel();
        final JLabel copyrightLabel = new JLabel(copyright);
        copyrightLabel.setFont(f2);
        copyrightLabel.setForeground(Color.black);
        copyrightLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        copyrightPanel.add(copyrightLabel);
        textPanel.add(copyrightPanel);
        
        for (int i = 0; i < info.length; ++i) {
            final JPanel infoPanel = new JPanel();
            final JLabel infoLabel = new JLabel(info[i]);
            infoLabel.setFont(f2);
            infoLabel.setForeground(Color.black);
            infoLabel.setHorizontalTextPosition(SwingConstants.CENTER);
            infoPanel.add(infoLabel);
            textPanel.add(infoPanel);
        }
        
        add(textPanel);
        
        if (logo != null) {
            final JPanel imagePanel = new JPanel(new BorderLayout());
            imagePanel.add(new javax.swing.JLabel(new javax.swing.ImageIcon(logo)));
            imagePanel.setBorder(BorderFactory.createLineBorder(Color.black));
            final JPanel imageContainer = new JPanel(new BorderLayout());
            imageContainer.add(imagePanel, BorderLayout.NORTH);
            add(imageContainer, BorderLayout.WEST);
        }
        
    }
    
}
