/*
 * TextAreaBox.java
 */

package de.pdv_systeme.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import org.jfree.ui.RefineryUtilities;

/**
 * Show a non-modal dialog with a (non-editable) text area and an OK button.
 *
 * @author Martin Vorlnder <mv@pdv-systeme.de>
 */
public class TextAreaBox {
    
    JTextArea textArea;
    
    /** Creates a new instance of TextAreaBox.
     *
     * @param parent  the JFrame object parenting the message box
     * @param title  the dialog's window title
     * @param headline  a headline for the text area
     * @param rows  the number of rows for the text area
     * @param columns the number of columns for the text area
     */
    public TextAreaBox(
            JFrame parent,
            String title,
            String headline,
            int rows,
            int columns
            ) {
        textArea = new JTextArea(rows, columns);
        textArea.setEditable(false);
        JButton okButton = new JButton("OK");
        JOptionPane optionPane = new JOptionPane(
                new Object[] {headline, new JScrollPane(textArea)},
                JOptionPane.PLAIN_MESSAGE,
                JOptionPane.OK_CANCEL_OPTION, // or whatever...
                null, // no icon
                new Object[] {okButton},
                okButton
                );
        final JDialog textAreaDialog = optionPane.createDialog(parent, title);
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                textAreaDialog.dispose();
            }
        });
        RefineryUtilities.centerDialogInParent(textAreaDialog);
        
        // Show dialog in a separate thread (as it is modal)
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                textAreaDialog.show();
            }
        });
    }
    
    /**
     * Get the TextAreaBox' text area widget.
     * @return the text area widget
     */
    public JTextArea getTextArea() {
        return textArea;
    }
    
}
