/*
 * TDCfileTest.java
 * JUnit based test
 */

package de.pdv_systeme.DataCollection;

import junit.framework.*;

import java.io.File;
import java.util.Date;
import java.util.List;

import org.jfree.data.time.TimeSeries;
import de.pdv_systeme.util.VmsDate;


/**
 * <code>TDCDataCollection</code> unit test
 * 
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class TDCfileTest extends TestCase {
    
    static String filename;
    static int allColumns;
    static int lastRows;
    static String lastColumn;
    static Date lastTime;
    static double lastValue;
    
    /** ECP/TDC file to use for testing */
    private static final String TEST_FILE_TDC
            = "C:\\work\\ecp_vmsspt_31mar2005.csv";
    
    public TDCfileTest(String testName) {
        super(testName);
    }
    
    protected void setUp() throws Exception {
        filename = TEST_FILE_TDC;
        allColumns = 656;
        lastRows = 287;
        lastColumn = "[XQP]Quota miss rate";
        lastTime = VmsDate.parse("31-MAR-2005 23:57:33.60");
        lastValue = 0.0;
    }
    
    protected void tearDown() throws Exception {
    }
    
    /**
     * suite method automatically generated by JUnit module
     *
     */
    public static Test suite() {
        TestSuite suite = new TestSuite("TDCfileTest");
        
        suite.addTest(new TDCfileTest("testFormat"));
        suite.addTest(new TDCfileTest("testLoad"));

        return suite;
    }
    
    // Add test methods here. The name must begin with 'test'. For example:
    // public void testHello() {}
    
    public void testFormat() {
        try {
            assertTrue(TDCDataCollection.testFormat(new File(filename)));
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }
    
    public void testLoad() {
        TDCDataCollection tdc = null;
        
        try {
            tdc = new TDCDataCollection(new File(filename));
        } catch (Exception e) {
            fail(e.getMessage());
        }
        assertTrue(tdc != null);
        
        assertEquals("Format", "TDC", tdc.getFormat());
        
        assertEquals("Filename", filename, tdc.getFilename());
        
        try {
            tdc.loadData();
        } catch (Exception e) {
            fail(e.getMessage());
        }
        
        List names = tdc.getAllItemNames();
        int columns = names.size();
        assertEquals("Number of columns", allColumns, columns);
        assertEquals("Last column name", lastColumn, names.get(columns-1));
        
        TimeSeries lastValues = tdc.getData((String) names.get(columns-1));
        int values = lastValues.getItemCount();
        assertEquals("Number of data records", lastRows, values);
        assertEquals("Last column value", lastValue,
                lastValues.getDataItem(values-1).getValue().doubleValue(), 0.0);
        
        /* have to cut of the fractional seconds,
         * because RegularTimePeriod sets them to .999
         * which can't be set through a VmsDate */
        assertEquals("Last timestamp", (int)(lastTime.getTime() / 1000),
                (int)(lastValues.getDataItem(values-1).getPeriod().getEnd().getTime() / 1000));
    }
}
