/*
 * TLCfileTest.java
 * JUnit based test
 */

package de.pdv_systeme.DataCollection;

import junit.framework.*;

import java.io.File;
import java.util.Date;
import java.util.List;

import org.jfree.data.time.TimeSeries;
import de.pdv_systeme.util.VmsDate;


/**
 * <code>TLCDataCollection</code> unit test
 * 
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class TLCfileTest extends TestCase {
    static String filename;
    static int allColumns;
    static int allRows;
    static String lastColumn;
    static Date lastTime;
    static double lastValue;
    
    /** TLC file to use for testing */
    private static final String TEST_FILE_TLC
            = "C:\\work\\t4_max_01apr2005_0000_0000.csv";
    
    public TLCfileTest(String testName) {
        super(testName);
    }
    
    protected void setUp() throws Exception {
        filename = TEST_FILE_TLC;
        allColumns = 184;
        allRows = 1432;
        lastColumn = "[XFC]Write IOs/Sec";
        lastTime = VmsDate.parse(" 2-APR-2005 00:00:13.13");
        lastValue = 0.0;
    }
    
    protected void tearDown() throws Exception {
    }
    
    /**
     * suite method automatically generated by JUnit module
     *
     */
    public static Test suite() {
        TestSuite suite = new TestSuite("TLCfileTest");
        
        suite.addTest(new TLCfileTest("testFormat"));
        suite.addTest(new TLCfileTest("testLoad"));
        
        return suite;
    }
    
    // Add test methods here. The name must begin with 'test'. For example:
    // public void testHello() {}
    
    public void testFormat() {
        try {
            assertTrue(TLCDataCollection.testFormat(new File(filename)));
        } catch (Exception e) {
            fail(e.getMessage());
        }
    }
    
    public void testLoad() {
        TLCDataCollection tlc = null;

        try {
            tlc = new TLCDataCollection(new File(filename), false);
        } catch (Exception e) {
            fail(e.getMessage());
        }
        assertTrue(tlc != null);
        
        assertEquals("Format", "TLC", tlc.getFormat());

        assertEquals("Filename", filename, tlc.getFilename());
        
        try {
            tlc.loadData();
        } catch (Exception e) {
            fail(e.getMessage());
        }
        
        List names = tlc.getAllItemNames();
        int columns = names.size();
        assertEquals("Number of columns", allColumns, columns);
        assertEquals("Last column name", lastColumn, names.get(columns-1));
        
        TimeSeries lastValues = tlc.getData((String) names.get(columns-1));
        int values = lastValues.getItemCount();
        assertEquals("Number of data records", allRows, values);
        assertEquals("Last column value",
                lastValue, lastValues.getDataItem(values-1).getValue().doubleValue(), 0.0);
        
        /* have to cut of the fractional seconds,
         * because RegularTimePeriod sets them to .999
         * which can't be set through a VmsDate */
        assertEquals("Last timestamp", (int)(lastTime.getTime() / 1000),
                (int)(lastValues.getDataItem(values-1).getPeriod().getEnd().getTime() / 1000));
    }
}
