/*
 * MyCollectionsTest.java
 * JUnit based test
 *
 * Created on 5. Mrz 2006, 06:24
 */

package de.pdv_systeme.util;

import java.util.Collections;
import junit.framework.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Test suite for the MyCollection methods
 *
 * @author <a href="mailto:mv@pdv-systeme.de">Martin Vorlnder</a>
 */
public class MyCollectionsTest extends TestCase {

    private static final int TESTSIZE = 101; /* odd number */

    private List listOdd, listEven;
    private Object[] arrOdd, arrEven;
    
    public MyCollectionsTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        assertEquals("TESTSIZE must be odd.", 1, TESTSIZE % 2);

        listOdd = new ArrayList(TESTSIZE);
        for (int i = 0; i < TESTSIZE; ++i)
            listOdd.add(new Double(Math.random()));
        arrOdd = listOdd.toArray(new Double[TESTSIZE]);
        Arrays.sort(arrOdd);
        
        listEven = new ArrayList(TESTSIZE+1);
        for (int i = 0; i < TESTSIZE+1; ++i)
            listEven.add(new Double(Math.random()));
        arrEven = listEven.toArray(new Double[TESTSIZE+1]);
        Arrays.sort(arrEven);
    }

    protected void tearDown() throws Exception {
        listOdd.clear();
        listEven.clear();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(MyCollectionsTest.class);

        suite.addTest(new MyCollectionsTest("testSelectOdd"));
        suite.addTest(new MyCollectionsTest("testMedianOdd"));

        suite.addTest(new MyCollectionsTest("testSelectEven"));
        suite.addTest(new MyCollectionsTest("testMedianEven"));

        return suite;
    }

    /**
     * Test of select method, of class de.pdv_systeme.util.MyCollections.
     */
    public void testSelectOdd() {
        for (int i = 0; i < TESTSIZE; ++i) {
            // System.out.println("testSelect odd "+String.valueOf(i+1));

            Object result = MyCollections.select(listOdd, i+1);
            assertEquals(arrOdd[i], result);
        }
    }

    /**
     * Test of select method, of class de.pdv_systeme.util.MyCollections.
     */
    public void testSelectEven() {
        for (int i = 0; i < TESTSIZE+1; ++i) {
            // System.out.println("testSelect even "+String.valueOf(i+1));

            Object result = MyCollections.select(listEven, i+1);
            assertEquals(arrEven[i], result);
        }
    }

    /**
     * Test of median method, of class de.pdv_systeme.util.MyCollections.
     */
    public void testMedianOdd() {
        // System.out.println("testMedian odd");
        
        Object result = MyCollections.median(listOdd);
        assertEquals(arrOdd[arrOdd.length>>1], result);
    }
    
    /**
     * Test of median method, of class de.pdv_systeme.util.MyCollections.
     */
    public void testMedianEven() {
        // System.out.println("testMedian even");
        
        Object result = MyCollections.median(listEven);
        assertEquals(arrEven[arrEven.length>>1], result);
    }
    
}
